<#
CodeTwoDiagnostics.ps1
Run this script to collect some information about local server and Exchange organization.
To run this script execute following:
1. Log in to your Exchange Server using your Domain Admin account.
2. Run the Exchange Management Shell.
3. Go to the script location, e.g. "cd c:\users\Administrator\desktop\".
4. Run the script by executing this command: ".\CodeTwoDiagnostics.ps1".
5. Go to the location with script results (will be displayed at the end of script execution).
6. Archive folder CodeTwoDiagnostics and send it to CodeTwo Support Team along with other files requested by the support team.
#>

# The below info will be displayed after executing the script

cls;
Write-Host "`nCodeTwo diagnostic script." -foregroundcolor yellow;
Write-Host "`nIt may take a minute or two to complete execution of the script.`n" -foregroundcolor yellow;
Write-Host " * Are you logged in as a Domain Admin?                           " backgroundcolor red -foregroundcolor yellow;
Write-Host " * Is this script run in Exchange Management Shell?               `n" backgroundcolor red -foregroundcolor yellow;
Write-Host " * Have you changed the logging level of CodeTwo Agent to detailed`n   and sent test messages as per http://www.codetwo.com/kb/278 ?  " backgroundcolor red -foregroundcolor yellow;
Write-Host "   This must be done PRIOR TO RUNNING THIS SCRIPT!                "backgroundcolor red -foregroundcolor yellow;
Write-Host "`nDo you meet the above requirements? Type in YES to start.`n" -foregroundcolor yellow;

# Defying a function that will be used to automatically compress the diagnostic files.

function Add-Zip {
    param([string]$zipfilename);

    if(-not (test-path($zipfilename)))
    {
        set-content $zipfilename ("PK" + [char]5 + [char]6 + ("$([char]0)" * 18));
        (dir $zipfilename).IsReadOnly = $false;
    }

    $shellApplication = new-object -com shell.application;
    $zipPackage = $shellApplication.NameSpace($zipfilename);

    foreach($file in $input) 
    { 
            $zipPackage.CopyHere($file.FullName);
            Write-Host "." -nonewline;
            Start-sleep -milliseconds 500;
    }
}

# Awaiting for the user input confirming running the script with the domain admin permissions and in the Exchange Management Shell.

$answer = Read-Host "?";

if($answer -eq "YES") {

# Defying diagnostic results folder name (C2Diag-<date_and_time> in $env:userprofile, e.g. C:\users\Administrator\C2Diag-20130925124500).

$timestamp = Get-Date -Format s | foreach {$_ -replace ":", ""} | foreach {$_ -replace "-", ""};
$exportpath = "$env:userprofile\C2Diag-$timestamp";

# Creating diagnostic results folder.

if ((Test-Path -path $exportpath\) -ne $True) {
	Write-Host "`nCreating directory $exportpath" -foregroundcolor yellow;
	New-Item $exportpath -itemType directory | Out-Null;
}

Write-Host "Done.`n" -foregroundcolor yellow;
Write-Host "Gathering system details." -foregroundcolor yellow;

# try-catch-finally feature implemented to catch errors that are results of running script in the environment that does not meet the requirements.

try {

    # Checking the name of currently logged user.

    "Currently logged user is: " | Out-File  $exportpath\report.txt;
    [System.Security.Principal.WindowsIdentity]::GetCurrent().Name | Out-File $exportpath\report.txt -append -ErrorAction Stop;
    "" | Out-File  $exportpath\report.txt -append;

    # Checking who has local admin permissions [to compare with currently logged user to find out if it has appropriate permissions]

    "Local admins group: " | Out-File  $exportpath\report.txt -append;

    $admins = Get-WmiObject win32_groupuser computer $env:computername |? {$_.groupcomponent like '*"Administrators"'} -ErrorAction Stop
    $admins |% {  
    $_.partcomponent match .+Domain\=(.+)\,Name\=(.+)$ > $nul  
    $matches[1].trim('"') + \ + $matches[2].trim('"')  
    } | Out-File  $exportpath\report.txt -append;
    "" | Out-File  $exportpath\report.txt -append;

    # Checking what is the hostname.

    "Hostname is: " | Out-File  $exportpath\report.txt -append;
    $env:computername | Out-File  $exportpath\report.txt -append -ErrorAction Stop;
    "" | Out-File  $exportpath\report.txt -append;

    # Checking the Windows version.

    "OS version is: " | Out-File  $exportpath\report.txt -append;
    (Get-WmiObject -class Win32_OperatingSystem).Caption | Out-File  $exportpath\report.txt -append -ErrorAction Stop;
    "" | Out-File  $exportpath\report.txt -append;

    # Listing Exchange Servers and their roles [to get a full view of mail processing]

    "Exchange servers in organization: " | Out-File  $exportpath\report.txt -append;
    Get-ExchangeServer | select name, serverrole, edition, admindisplayversion | fl | Out-File  $exportpath\report.txt -append -ErrorAction Stop;

    # Checking the configuration for the remote domains in your organization [important in cases where email format may be changing]

    "Remote domain settings: " | Out-File $exportpath\report.txt -append;
    Get-RemoteDomain | fl | Out-File $exportpath\report.txt -append -ErrorAction Stop;
    "" | Out-File  $exportpath\report.txt -append;

    # Listing Exchange Transport agents on the local server [to exclude possible agents conflicts and get a full view of mail processing]

    "Exchange transport agents on local server: " | Out-File  $exportpath\report.txt -append;
    Get-TransportAgent | Out-File  $exportpath\report.txt -append -ErrorAction Stop;
    "" | Out-File  $exportpath\report.txt -append;

    # Listing Exchange Transport rules on the local server [to exclude possible rules conflicts and get a full view of mail processing]

    "List of Exchange Transport rules: " | Out-File $exportpath\report.txt -append;
    Get-TransportRule | Out-File $exportpath\report.txt -append -ErrorAction Stop;

    # Finding the EWS addresses [important for Sent Items Update configuration]

    "EWS addresses: " | Out-File $exportpath\report.txt -append;
    Get-WebServicesVirtualDirectory |Select name, *url* | fl | Out-File $exportpath\report.txt -append -ErrorAction Stop;

    "-EOF-" | Out-File  $exportpath\report.txt -append;
    Write-Host "Done.`n" -foregroundcolor yellow;
    }

catch {

    # a popup window will be generated to inform the user about problems encountered executing the above commands.

    [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") | Out-Null;
    [System.Windows.Forms.MessageBox]::Show("One or more operations were not succefull.`nThis was probably caused by executing the script improperly.`nThe script will continue to work but the results will be incomplete. It is strongly advised you re-run it making sure you meet the requirements below:`n`n * You are logged in as Domain Admin`n`n * You are running this script in Exchange Management Shell`n`n * You are running this script directly on your Exchange Server`nwhere CodeTwo software is installed.`n", " - WARNING! - ", "OK", "Exclamation") | Out-Null;
    Write-Host "FAILED.`n" -foregroundcolor red;
    }

finally {}

Write-Host "Searching for CodeTwo software installations and copying diagnostic files." -foregroundcolor yellow;

# Defying variables that will indicate wheter the script finds wanted files or not.

$founddiagsrules = 0;
$founddiagsagentlogs = 0;

# Searching for the CodeTwo: log files, used licenses info, license key file - this section applies to Windows Server 2003 only.

if ( $(Get-WmiObject -Class win32_operatingsystem).version -like "5*" ) {
	if ((Test-Path -path "$exportpath\eventlog_application.evt") -eq $True) {
	    Remove-Item "$exportpath\eventlog_application.evt" -force
	}
	$logFile = Get-WmiObject Win32_NTEventlogFile | Where-Object {$_.logfilename -eq "Application"};
	$logFile.PSBase.Scope.Options.EnablePrivileges = $true;
	$logFile.backupeventlog("$exportpath\eventlog_application.evt");
	$logFile.PSBase.Scope.Options.EnablePrivileges = $false;

	if ((Test-Path -path "$exportpath\eventlog_system.evt") -eq $True) {
	    Remove-Item "$exportpath\eventlog_system.evt" -force
	}
	$logFile1 = Get-WmiObject Win32_NTEventlogFile | Where-Object {$_.logfilename -eq "System"};
	$logFile1.PSBase.Scope.Options.EnablePrivileges = $true;
	$logFile1.backupeventlog("$exportpath\eventlog_system.evt");
	$logFile1.PSBase.Scope.Options.EnablePrivileges = $false;
    $countpaths = 0;
    $source = (
    "$env:programfiles\CodeTwo\ExchangeRules\Logs",
    "C:\Documents and Settings\NetworkService\Application Data\CodeTwo\ExchangeRules 2007\Logs",
    "C:\Documents and Settings\NetworkService\Application Data\CodeTwo\CodeTwo Exchange Rules 2007\AgentLogs",
    "C:\Documents and Settings\NetworkService\Application Data\CodeTwo\CodeTwo Exchange Rules Pro\AgentLogs",
    "C:\Documents and Settings\CodeTwo Update Agent\Application Data\CodeTwo\SentItems Updater\Logs",
    "$env:alluserprofile\Application Data\CodeTwo SentItems Updater\Logs",
    "$env:alluserprofile\Application Data\CodeTwo SentItems Updater Pro\Logs",
    "$env:programfiles\CodeTwo\ExchangeRules\Licensing.log",
    "C:\Documents and Settings\NetworkService\Application Data\CodeTwo\ExchangeRules 2007\Licensing.log",
    "C:\Documents and Settings\NetworkService\Application Data\CodeTwo\CodeTwo Exchange Rules 2007\LicensingInfo.log",
    "C:\Documents and Settings\NetworkService\Application Data\CodeTwo\CodeTwo Exchange Rules Pro\LicensingInfo.log",
    "$env:programfiles\CodeTwo\ExchangeRules\key.dat",
    "$env:programfiles\CodeTwo\CodeTwo ExchangeRules 2007\ExchangeRules.lf",
    "$env:alluserprofile\Application Data\CodeTwo Exchange Rules 2007\ExchangeRules.lf",
    "$env:alluserprofile\Application Data\CodeTwo Exchange Rules Pro\ExchangeRulesPro.lf",
    "C:\Documents and Settings\Administrator\Application Data\CodeTwo\CodeTwo Exchange Rules Pro\ClientLogs",
    "$env:alluserprofile\Application Data\CodeTwo Exchange Rules 2007\Rules",
    "$env:alluserprofile\Application Data\CodeTwo Exchange Rules Pro\Rules");
    $target = (
    "2003-3-Agentlogs",
    "2007-2-Agentlogs",
    "2007-3-Agentlogs",
    "PRO-1-Agentlogs",
    "2007-2-SIUlogs",
    "2007-3-SIUlogs",
    "PRO-1-SIUlogs",
    "2003-3-licensing",
    "2007-2-licensing",
    "2007-3-licensing",
    "PRO-1-licensing",
    "2003-3-lffile",
    "2007-2-lffile",
    "2007-3-lffile",
    "PRO-1-lffile",
    "PRO-1-Administration",
    "2007-3-Rules",
    "PRO-1-Rules");
    do {
        if ((Test-Path -path $source[$countpaths]) -eq $True) {
            Write-Host "Found" $source[$countpaths] -foregroundcolor green;
            $path = $target[$countpaths];
	    	New-Item $exportpath\$path -itemType directory | Out-Null;
            Copy-Item $source[$countpaths] -Destination $exportpath\$path -Recurse | Out-Null;
            if ($path -Match "Rules") {
                $founddiagsrules++;
            }
            if ($path -Match "Agentlogs") {
                $founddiagsagentlogs++;
            }
	    }
        $countpaths++;
    }
    until($countpaths -eq 18);
}

# Searching for the CodeTwo: log files, used licenses info, license key file - this section applies to Windows Server 2008 or newer.

if ( $(Get-WmiObject -Class win32_operatingsystem).version -like "6*" ) {
	if ((Test-Path -path $exportpath\eventlog_application.evtx) -eq $True) {
		Remove-Item $exportpath\eventlog_application.evtx -force
	}
	wevtutil epl application /ow:true $exportpath\eventlog_application.evtx;

	if ((Test-Path -path $exportpath\eventlog_system.evtx) -eq $True) {
		Remove-Item $exportpath\eventlog_system.evtx -force
	}
	wevtutil epl system /ow:true $exportpath\eventlog_system.evtx;
    $countpaths = 0;
    $source = (
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\ExchangeRules 2007\Logs",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\ExchangeRules 2010\Logs",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules 2007\AgentLogs",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules 2010\AgentLogs",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules 2013\AgentLogs",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules Pro\AgentLogs",
    "C:\Users\CodeTwo Update Agent\AppData\Roaming\CodeTwo\SentItems Updater\Logs",
    "$env:programdata\CodeTwo SentItems Updater\Logs",
    "$env:programdata\CodeTwo SentItems Updater Pro\Logs",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\ExchangeRules 2007\Licensing.log",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\ExchangeRules 2010\Licensing.log",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules 2007\LicensingInfo.log",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules 2010\LicensingInfo.log",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules 2013\LicensingInfo.log",
    "$env:windir\ServiceProfiles\NetworkService\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules Pro\LicensingInfo.log",
    "$env:programfiles\CodeTwo\CodeTwo ExchangeRules 2007\ExchangeRules.lf",
    "$env:programfiles\CodeTwo\CodeTwo ExchangeRules 2010\ExchangeRules.lf",
    "$env:programdata\CodeTwo Exchange Rules 2007\ExchangeRules.lf",
    "$env:programdata\CodeTwo Exchange Rules 2010\ExchangeRules.lf",
    "$env:programdata\CodeTwo Exchange Rules 2013\ExchangeRules.lf",
    "$env:programdata\CodeTwo Exchange Rules Pro\ExchangeRulesPro.lf",
    "$env:userprofile\AppData\Roaming\CodeTwo\CodeTwo Exchange Rules Pro\ClientLogs",
    "$env:programdata\CodeTwo Exchange Rules 2007\Rules",
    "$env:programdata\CodeTwo Exchange Rules 2010\Rules",
    "$env:programdata\CodeTwo Exchange Rules 2013\Rules",
    "$env:programdata\CodeTwo Exchange Rules Pro\Rules");
    $target = (
    "2007-2-Agentlogs",
    "2010-1-Agentlogs",
    "2007-3-Agentlogs",
    "2010-2-Agentlogs",
    "2013-1-Agentlogs",
    "PRO-1-Agentlogs",
    "ER-OLD-SIUlogs",
    "ER-NEW-SIUlogs",
    "PRO-1-SIUlogs",
    "2007-2-licensing",
    "2010-1-licensing",
    "2007-3-licensing",
    "2010-2-licensing",
    "2013-1-licensing",
    "PRO-1-licensing",
    "2007-2-lffile",
    "2010-1-lffile",
    "2007-3-lffile",
    "2010-2-lffile",
    "2013-1-lffile",
    "PRO-1-lffile",
    "PRO-1-Administration",
    "2007-3-Rules",
    "2010-2-Rules",
    "2013-1-Rules",
    "PRO-1-Rules");
    do {
        if ((Test-Path -path $source[$countpaths]) -eq $True) {
            Write-Host "Found" $source[$countpaths] -foregroundcolor green;
            $path = $target[$countpaths];
	    	New-Item $exportpath\$path -itemType directory | Out-Null;
            Copy-Item $source[$countpaths] -Destination $exportpath\$path -Recurse | Out-Null;
            if ($path -Match "Rules") {
                $founddiagsrules++;
            }
            if ($path -Match "Agentlogs") {
                $founddiagsagentlogs++;
            }
	    }
        $countpaths++;
    }
    until($countpaths -eq 26);
}

Write-Host "Done.`n" -foregroundcolor yellow;

# Info for the user about file compressing function execution.

Write-Host "Compressing files, please hold" -foregroundcolor yellow -nonewline;

# Creating importable rules file(s) of found rules folder(s).

foreach($Rulesfolder in Get-ChildItem $exportpath\*Rules) { 
    Get-ChildItem $Rulesfolder | Add-Zip "$Rulesfolder.zip";
    if ($Rulesfolder -Match "PRO") {
    Rename-Item -Path "$Rulesfolder.zip" -NewName "$Rulesfolder.c2saf"}
    else {
    Rename-Item -Path "$Rulesfolder.zip" -NewName "$Rulesfolder.ersaf"}
    }

# Removing rules folder(s) that are no longer needed.

Remove-Item $exportpath\*Rules -Recurse

# Compressing all files together.

Get-ChildItem $exportpath | Add-Zip $exportpath\Send-me-to-C2.zip;

# Removing all files and folders except for the .zip file.

Remove-Item $exportpath\* -Recurse -Exclude *.zip

Write-Host "`nDone.`n" -foregroundcolor yellow;

# A warning that will be displayed in the case the script did not find files it was looking for.

if ($founddiagsrules -eq 0 -or $founddiagsagentlogs -eq 0){
    $warning = "Warning!`n`nThe following files were not found in the default location!";
    $warningrules = " * Set of rules";
    $warningagentlogs = " * Agent log files";
    if ($founddiagsrules -eq 0) {
        $warning = $warning + "`n`n$warningrules";
        }
    if ($founddiagsagentlogs -eq 0) {
        $warning = $warning + "`n`n$warningagentlogs";
        }
    $warning = $warning + "`n`nPlease extract these files from the program manually`nas per http://www.codetwo.com/kb/278`n";
    Write-Host $warning backgroundcolor red -foregroundcolor yellow;
    [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms") | Out-Null;
    [System.Windows.Forms.MessageBox]::Show("$warning`n`nClick OK to continue.", " - WARNING! - ", "OK", "Exclamation") | Out-Null;
    }

# Info for the user what to do next upon script execution is complete.

Write-Host "`nWhat to do next?" backgroundcolor red -foregroundcolor yellow;
Write-Host "`n 1. Go to: $exportpath\" -foregroundcolor yellow;
Write-Host "`n 2. Copy there and compress all other files such as .msg examples, into .zip files and send to CodeTwo." -foregroundcolor yellow;
Write-Host "`n 3. If you already have a case open with CodeTwo please provide the ticket number in the subject line, e.g. [#12345]." -foregroundcolor yellow;
Write-Host "`nPlease allow 24h for analysis of the diagnostic files.`n" -foregroundcolor yellow;

# Opening a folder with the diagnostic files.

Invoke-Item $exportpath;
}

# Alternative script closing in the case a user gives other answer than "yes" in the beginning.

else {
Write-Host "`nScript execution cancelled by the user.`n" -foregroundcolor yellow;
}